@@ -19,13 +19,20 @@ if (typeof(window['yafowil']) == "undefined") yafowil = {};
                 datepicker_binder: yafowil.datepicker.binder
             });
         }
+        
+        // add binder to yafowil.widget.array hooks
+        if (typeof(window.yafowil['array']) != "undefined") {
+            $.extend(yafowil.array.hooks.add, {
+                datepicker_binder: yafowil.datepicker.binder
+            });
+        }
     });
     
     $.extend(yafowil, {
         
         datepicker: {
             
-            icon: '/++resource++yafowil.widget.datetime/images/cal16_16.gif',
+        	icon: '++resource++yafowil.widget.datetime/images/cal16_16.gif',
             
             binder: function(context) {
                 $('input.datepicker', context).datepicker({
